import re
from typing import Sequence

from epstein_files.util.constant.strings import PALM_BEACH, QUESTION_MARKS, VIRGIN_ISLANDS
from epstein_files.util.helpers.string_helper import remove_question_marks

Name = str | None

# Unknown / None
UNKNOWN = '(unknown)'
UNKNOWN_GIRL = 'unknown girl'

# Texting Names
ANDRZEJ_DUDA = 'Andrzej Duda or entourage'
ANIL_AMBANI = "Anil Ambani"
ANTHONY_SCARAMUCCI = "Anthony Scaramucci"
ARDA_BESKARDES = 'Arda Beskardes'
EVA = 'Eva (Dubin?)'
JEFFREY_EPSTEIN = 'Jeffrey Epstein'
JOI_ITO = 'Joi Ito'
LARRY_SUMMERS = 'Larry Summers'
MELANIE_WALKER = 'Melanie Walker'
MIROSLAV_LAJCAK = 'Miroslav Lajčák'
STACEY_PLASKETT = 'Stacey Plaskett'
SOON_YI_PREVIN = 'Soon-Yi Previn'
STEVE_BANNON = 'Steve Bannon'
STEVEN_SINOFSKY = 'Steven Sinofsky'
TERJE_ROD_LARSEN = 'Terje Rød-Larsen'

# Email Names - no trailing periods! (messes up regexes)
AL_SECKEL = 'Al Seckel'
ALAN_DERSHOWITZ = 'Alan Dershowitz'
ALAN_DLUGASH = 'Alan Dlugash'
ALIREZA_ITTIHADIEH = 'Alireza Ittihadieh'
AMANDA_ENS = 'Amanda Ens'
ANASTASIYA_SIROOCHENKO = 'Anastasiya Siroochenko'
ANDRES_SERRANO = 'Andres Serrano'
ANN_MARIE_VILLAFANA = 'Ann Marie Villafana'
ANAS_ALRASHEED = 'Anas Alrasheed'
ANTHONY_BARRETT = 'Anthony Barrett'
ANYA_RASULOVA = 'Anya Rasulova'
ARIANE_DE_ROTHSCHILD = 'Ariane de Rothschild'
AZIZA_ALAHMADI = 'Aziza Alahmadi'
BARBRO_C_EHNBOM = 'Barbro C. Ehnbom'
BARRY_J_COHEN = 'Barry J. Cohen'
BEN_GOERTZEL = 'Ben Goertzel'
BENNET_MOSKOWITZ = 'Bennet Moskowitz'
BILL_SIEGEL = 'Bill Siegel'
BOB_CROWE = 'Bob Crowe'
BRAD_EDWARDS = 'Brad Edwards'
BRAD_KARP = 'Brad Karp'
BRAD_WECHSLER = 'Brad Wechsler'
BRYAN_BISHOP = 'Bryan Bishop'
BORIS_NIKOLIC = 'Boris Nikolic'
CAROLYN_RANGEL = 'Carolyn Rangel'
CECILE_DE_JONGH = 'Cecile de Jongh'
CECILIA_STEEN = 'Cecilia Steen'
CELINA_DUBIN = 'Celina Dubin'
CHRISTINA_GALBRAITH = 'Christina Galbraith'  # Works with Tyler Shears on reputation stuff
DANGENE_AND_JENNIE_ENTERPRISE = 'Dangene and Jennie Enterprise'
DANIEL_SABBA = 'Daniel Sabba'
DANIEL_SIAD = 'Daniel Siad'
DANNY_FROST = 'Danny Frost'
DARREN_INDYKE = 'Darren Indyke'
DAVID_BLAINE = 'David Blaine'
DAVID_FISZEL = 'David Fiszel'
DAVID_HAIG = 'David Haig'
DAVID_INGRAM = 'David Ingram'
DAVID_MITCHELL = 'David Mitchell'
DAVID_SCHOEN = 'David Schoen'
DAVID_STERN = 'David Stern'
DEBBIE_FEIN = 'Debbie Fein'
DEEPAK_CHOPRA = 'Deepak Chopra'
DIANE_ZIMAN = 'Diane Ziman'
DONALD_NORMAN = 'Donald Norman'
DONALD_TRUMP = 'Donald Trump'
EDUARDO_ROBLES = 'Eduardo Robles'
EDWARD_JAY_EPSTEIN = 'Edward Jay Epstein'
EDWARD_ROD_LARSEN = 'Edward Rod Larsen'
EHUD_BARAK = 'Ehud Barak'
ERIC_ROTH = 'Eric Roth'
ETIENNE_BINANT = 'Etienne Binant'
EVA_DUBIN = 'Eva Dubin'
FAITH_KATES = 'Faith Kates'
FRANCESCA_HALL = 'Francesca Hall'
FRED_HADDAD = 'Fred Haddad'
GERALD_BARTON = 'Gerald Barton'
GERALD_LEFCOURT = 'Gerald Lefcourt'
GHISLAINE_MAXWELL = 'Ghislaine Maxwell'
GINO_YU = 'Gino Yu'
GLENN_DUBIN = 'Glenn Dubin'
GORDON_GETTY = 'Gordon Getty'
GWENDOLYN_BECK = 'Gwendolyn Beck'         # https://www.lbc.co.uk/article/who-gwendolyn-beck-epstein-andrew-5HjdN66_2/
HAMAD_BIN_JASSIM = 'Hamad bin Jassim al-Thani'
HEATHER_MANN = 'Heather Mann'
IAN_OSBORNE = 'Ian Osborne'
INTELLIGENCE_SQUARED = 'Intelligence Squared'
JACK_GOLDBERGER = 'Jack Goldberger'
JACK_SCAROLA = 'Jack Scarola'
JACKIE_PERCZEK = 'Jackie Perczek'
JAMES_HILL = 'James Hill'
JANUSZ_BANASIAK = 'Janusz Banasiak'
JAY_LEFKOWITZ = 'Jay Lefkowitz'
JEAN_HUGUEN = 'Jean Huguen'
JEAN_LUC_BRUNEL = 'Jean Luc Brunel'
JEFF_FULLER = 'Jeff Fuller'
JENNIE_SAUNDERS = 'Jennie Saunders'
JENNIFER_JACQUET = 'Jennifer Jacquet'
JENNIFER_KALIN = 'Jennifer Kalin'
JEREMY_RUBIN = 'Jeremy Rubin'             # Bitcoin dev
JES_STALEY = 'Jes Staley'
JESSICA_CADWELL = 'Jessica Cadwell'       # Paralegal?
JIDE_ZEITLIN = 'Jide Zeitlin'
JOHN_BROCKMAN = "John Brockman"
JOHN_PAGE = 'John Page'
JOHNNY_EL_HACHEM = 'Johnny el Hachem'
JOJO_FONTANILLA = 'Jojo Fontanilla'
JONATHAN_FARKAS = 'Jonathan Farkas'
JOSCHA_BACH = 'Joscha Bach'
JP_MORGAN_USGIO = 'us.gio@jpmorgan.com'
KARIM_WADE = 'Karim Wade'
KATHERINE_KEATING = 'Katherine Keating'
KATHRYN_RUEMMLER = 'Kathryn Ruemmler'
KEN_JENNE = 'Ken Jenne'
KEN_STARR = 'Ken Starr'
KENNETH_E_MAPP = 'Kenneth E. Mapp'
LANDON_THOMAS = 'Landon Thomas Jr'
LAWRANCE_VISOSKI = 'Lawrance Visoski'
LAWRENCE_KRAUSS = 'Lawrence Krauss'
LEON_BLACK = 'Leon Black'
LESLEY_GROFF = 'Lesley Groff'
LILLY_SANCHEZ = 'Lilly Sanchez'
LINDA_STONE = 'Linda Stone'
LISA_NEW = 'Lisa New'
LISA_RANDALL = 'Lisa Randall'
LYN_FONTANILLA = 'Lyn Fontanilla'
MANUELA_MARTINEZ = 'Manuela Martinez'
MARC_LEON = 'Marc Leon'
MARIANA_IDZKOWSKA = 'Mariana Idźkowska'
MARK_EPSTEIN = 'Mark Epstein'
MARK_TRAMO = 'Mark Tramo'
MARTIN_NOWAK = 'Martin Nowak'
MARTIN_WEINBERG = "Martin Weinberg"
MARIA_PRUSAKOVA = 'Maria Prusakova'
MASAYOSHI_SON = 'Masayoshi Son'
MASHA_DROKOVA = 'Masha Drokova'
MATTHEW_HILTZIK = 'Matthew Hiltzik'
MELANIE_SPINELLA = 'Melanie Spinella'
MERWIN_DELA_CRUZ = 'Merwin Dela Cruz'
MICHAEL_BUCHHOLTZ = 'Michael Buchholtz'
MICHAEL_MILLER = 'Michael Miller'
MICHAEL_SITRICK = 'Michael Sitrick'
MICHAEL_WOLFF = "Michael Wolff"
MOHAMED_WAHEED_HASSAN = 'Mohamed Waheed Hassan'
MORTIMER_ZUCKERMAN = 'Mortimer Zuckerman'
MOSHE_HOFFMAN = 'Moshe Hoffman'
NADIA_MARCINKO = 'Nadia Marcinko'
NEAL_KASSELL = 'Neal Kassell'
NICHOLAS_RIBIS = 'Nicholas Ribis'
NILI_PRIELL_BARAK = 'Nili Priell Barak'
NOAM_CHOMSKY = 'Noam Chomsky'
NORMAN_D_RAU = 'Norman D. Rau'
OLIVIER_COLOM = 'Olivier Colom'
PAUL_BARRETT = 'Paul Barrett'
PAUL_KRASSNER = 'Paul Krassner'
PAUL_MORRIS = 'Paul Morris'
PAUL_PROSPERI = 'Paul Prosperi'
PAULA_HEIL_FISHER = f"Paula Heil Fisher"  # the last email about opera lines up but if Fisher was supposedly w/Epstein at Bear Stearns the timeline is a bit weird for her to call him "Unc"
PEGGY_SIEGAL = 'Peggy Siegal'
PETER_ATTIA = 'Peter Attia'
PETER_MANDELSON = 'Peter Mandelson'
PETER_THIEL = 'Peter Thiel'
PRINCE_ANDREW = 'Prince Andrew'
PUREVSUREN_LUNDEG = 'Purevsuren Lundeg'
RAAFAT_ALSABBAGH = 'Raafat Alsabbagh'
REINALDO_AVILA_DA_SILVA = 'Reinaldo Avila Da Silva'
REID_HOFFMAN = 'Reid Hoffman'
REID_WEINGARTEN = 'Reid Weingarten'
RENATA_BOLOTOVA = 'Renata Bolotova'
RICHARD_KAHN = 'Richard Kahn'
ROBERT_D_CRITTON_JR = 'Robert D. Critton Jr.'
ROBERT_LAWRENCE_KUHN = 'Robert Lawrence Kuhn'
ROBERT_TRIVERS = 'Robert Trivers'
ROGER_SCHANK = 'Roger Schank'
ROSS_GOW = 'Ross Gow'
SAMUEL_LEFF = 'Samuel Leff'
SCOTT_J_LINK = 'Scott J. Link'
SEAN_BANNON = 'Sean Bannon'
SHAHER_ABDULHAK_BESHER = f'Shaher Abdulhak Besher'
STANLEY_POTTINGER = 'Stanley Pottinger'
STEPHEN_HANSON = 'Stephen Hanson'
STEVEN_HOFFENBERG = 'Steven Hoffenberg'
STEVEN_PFEIFFER = 'Steven Pfeiffer'
SULTAN_BIN_SULAYEM = 'Sultan Ahmed Bin Sulayem'
SVETLANA_POZHIDAEVA = 'Svetlana Pozhidaeva'
TERRY_KAFKA = 'Terry Kafka'
THANU_BOONYAWATANA = 'Thanu Boonyawatana'
THORBJORN_JAGLAND = 'Thorbjørn Jagland'
TOM_BARRACK = 'Tom Barrack'
TOM_PRITZKER = 'Tom Pritzker'
TONJA_HADDAD_COLEMAN = 'Tonja Haddad Coleman'
TYLER_SHEARS = 'Tyler Shears'  # Reputation manager, like Al Seckel
VINCENZO_IOZZO = 'Vincenzo Iozzo'
VINIT_SAHNI = 'Vinit Sahni'
WOODY_ALLEN = 'Woody Allen'
ZUBAIR_KHAN = 'Zubair Khan'

# DOJ files emails
ADAM_BACK = 'Adam Back'
ALEKSANDRA_KARPOVA = 'Aleksandra Karpova'
ALISON_J_NATHAN = 'Alison J. Nathan'
AMIR_TAAKI = 'Amir Taaki'
ANDREW_FARKAS = 'Andrew Farkas'
ANDREW_MCCORMACK = 'Andrew McCormack'
ANNA_KASATKINA = 'Anna Kasatkina'
ARIANNA_SIMPSON = 'Arianna Simpson'
AUDREY_STRAUSS = 'Audrey Strauss'
AUSTIN_HILL = 'Austin Hill'
BARNABY_MARSH = 'Barnaby Marsh'
BELLA_KLEIN = 'Bella Klein'
BILL_RICHARDSON = 'Bill Richardson'
BOBBI_C_STERNHEIM = 'Bobbi C. Sternheim'
BOBBY_KOTICK = 'Bobby Kotick'
BROCK_PIERCE = 'Brock Pierce'
CHRIS_POOLE = 'Chris Poole'
CHRISTIAN_EVERDELL = 'Christian Everdell'
CHRISTOPHER_DILORIO = 'Christopher Dilorio'
DANIEL_SCHMACHTENBERGER = 'Daniel Schmachtenberger'
DAPHNE_WALLACE = 'Daphne Wallace'
DASHA_GRUPMAN = 'Dasha Grupman'
DAVID_L_NEUHAUSER = 'David L. Neuhauser'
DAVID_RODGERS = 'David Rodgers'
DOUGLAS_WIGDOR = 'Douglas Wigdor'
ED_BOYLE = 'Ed Boyle'
ELON_MUSK = 'Elon Musk'
EMAD_HANNA = 'Emad Hanna'
FILIPA_PEROVIC = 'Filipa Perovic'
FAWZI_SIAM = 'Fawzi Siam'
GANBAT_CHULUUNKHUU = 'Ganbat Chuluunkhuu'
GAVIN_ANDRESEN = 'Gavin Andresen'
GREG_WYLER = 'Greg Wyler'
HASSAN_JAMEEL = 'Hassan Jameel'
HENRY_JARECKI = 'Henry Jarecki'
HONGBO_ROBERT_BAO = 'Hongbo Robert Bao'
HOSAIN_RAHMAN = 'Hosain Rahman'
HOWARD_LUTNICK = 'Howard Lutnick'
IAN_ODONNELL = "Ian O'Donnell"
JAMES_CAYNE = 'James Cayne'
JAMES_FITZGERALD = 'James Fitzgerald'
JAMES_TAGG = 'James Tagg'
JASON_CALACANIS = 'Jason Calacanis'
JEM_BENDELL = 'Jem Bendell'
JOSHUA_FINK = 'Joshua Fink'
JULIA_SANTOS = 'Julia Santos'
KARYNA_SHULIAK = 'Karyna Shuliak'
KIMBAL_MUSK = 'Kimbal Musk'
KIRA_DIKHTYAR = 'Kira Dikhtyar'
LAURA_A_MENNINGER = 'Laura A. Menninger'
LORENZO_DE_MEDICI = 'Lorenzo de Medici'
LUKE_D_THORBURN = 'Luke D. Thorburn'
MADARS_VIRZA = 'Madars Virza'
MELANIE_PHILLIPS = 'Melanie Phillips'
MICHAEL_FOWLER = 'Michael Fowler'
NICOLE_JUNKERMANN = 'Nicole Junkermann'
OLGA_PONOMAR_BECKER = 'Olga Ponomar-Becker'
OLIVER_GOODENOUGH = 'Oliver Goodenough'
PERKINS_COIE = 'Perkins Coie'
PHILIP_ROSEDALE = 'Philip Rosedale'
RAMSEY_ELKHOLY = 'Ramsey Elkholy'
RASSECK_BOURGI = 'Rasseck Bourgi'
SANITA = 'Sanita'
SETH_LLOYD = 'Seth Lloyd'
STACEY_RICHMAN = 'Stacey Richman'
STEVEN_VICTOR_MD = 'Steven Victor MD'
STORY_COWLES = 'Story Cowles'
SUE = 'Sue'
TANCREDI_MARCHIOLO = 'Tancredi Marchiolo'
UMAR_DZHABRAILOV = 'Umar Dzhabrailov'
WHITFIELD_DIFFIE = 'Whitfield Diffie'
YUKO_BARNABY = 'Yuko Barnaby'
YULIA_DOROKHINA = 'Yulia Dorokhina'
YURI_MILNER = 'Yuri Milner'

# No communications but name is in the files
BEN_LAWSKY = 'Ben Lawsky'
BILL_GATES = 'Bill Gates'
CHAMATH_PALIHAPITIYA = 'Chamath Palihapitiya'
DONALD_TRUMP = 'Donald Trump'
GEORGE_VRADENBURG = 'George Vradenburg'
GLORIA_ALLRED = 'Gloria Allred'
HENRY_HOLT = 'Henry Holt'  # Actually a company?
IVANKA = 'Ivanka'
JAMES_PATTERSON = 'James Patterson'
JARED_KUSHNER = 'Jared Kushner'
JEANNE_M_CHRISTENSEN = 'Jeanne M. Christensen'
JEFFREY_WERNICK = 'Jeffrey Wernick'
JULIE_K_BROWN = 'Julie K. Brown'
KARIM_SADJADPOUR = 'KARIM SADJADPOUR'.title()
MICHAEL_J_BOCCIO = 'Michael J. Boccio'
MILES_GUO = 'Miles Guo'
NAOMI_CAMPBELL = 'Naomi Campbell'
NERIO_ALESSANDRI = 'Nerio Alessandri (Founder and Chairman of Technogym S.p.A. Italy)'
PAUL_G_CASSELL = 'Paul G. Cassell'
RUDY_GIULIANI = 'Rudy Giuliani'
SERGEY_BELYAKOV = 'Sergey Belyakov'
STEVE_WYNN = 'Steve Wynn'
TED_LEONSIS = 'Ted Leonsis'
TULSI_GABBARD = 'Tulsi Gabbard'
VIRGINIA_GIUFFRE = 'Virginia Giuffre'

# Organizations
ALL_IN_PODCAST = 'All-In Podcast'
ASIA_GATEWAY = 'Asia Gateway'
ATORUS = 'ATorus'
ATT_COURT_APPEARANCE_TEAM = 'AT&T Court Appearance Team'
BANQUE_HAVILLAND = 'Banque Havilland'
BEAR_STEARNS = 'Bear Stearns'
BLOCKCHAIN_CAPITAL = 'Blockchain Capital'
BLOCKSTREAM = 'Blockstream'
BOFA = 'BofA'
BOFA_MERRILL = f'{BOFA} / Merrill Lynch'
BUREAU_OF_PRISONS = 'Bureau of Prisons'
CANTOR = 'Cantor Fitzgerald'
CANTOR_URRAMOOR = 'Cantor Urramoor Asset Management'
CARBYNE = 'Carbyne'
CLIFFORD_CHANCE = 'Clifford Chance'
COINBASE = 'Coinbase'
CRYPTO_CURRENCY_PARTNERS_II = 'Crypto Currency Partners II'
CRYPTO_PR_LAB = 'Crypto PR Lab'
DEUTSCHE_BANK = 'Deutsche Bank'
DOJ = 'DOJ'
EDMOND_DE_ROTHSCHILD = 'Edmond de Rothschild'
ELECTRON_CAPITAL_PARTNERS = 'Electron Capital Partners'
EPSTEIN_FOUNDATION = 'Jeffrey Epstein VI Foundation'
EXPRESSCOIN = 'ExpressCoin'
FBI = 'FBI'
GOLDMAN_SACHS = 'Goldman Sachs'
GOOGLE_PLUS = 'Google Plus'
HEDOSOPHIA = 'Hedosophia'
INSIGHTS_POD = f"InsightsPod"  # Zubair bots
INTERNATIONAL_PEACE_INSTITUTE = 'International Peace Institute'
JP_MORGAN = 'JP Morgan'
KYARA_INVESTMENT = 'Kyara Investment'
LAWRENCE_KRAUSS_ASU_ORIGINS = f"{LAWRENCE_KRAUSS}'s ASU Origins Project"
MC2_MODEL_MGMT = 'MC2 Model Management'
MERCANTILE_GLOBAL_HOLDINGS = 'Mercantile Global Holdings'
MIT_MEDIA_LAB = 'MIT Media Lab'
MOUNT_SINAI = 'Mount Sinai'
NEXT_MANAGEMENT = 'Next Management LLC'
NOBEL_CHARITABLE_TRUST = 'Nobel Charitable Trust'
OFFICE_OF_THE_DEPUTY_ATTORNEY_GENERAL = 'Office of the Deputy Attorney General'
OSBORNE_LLP = f"{IAN_OSBORNE} & Partners"  # Ian Osborne's PR firm
PALM_BEACH_POLICE = f"{PALM_BEACH} Police"
ROTHSTEIN_ROSENFELDT_ADLER = "Rothstein Rosenfeldt Adler"  # and a crook
SDNY = 'SDNY'
SECURITIES_AND_EXCHANGE_COMMISSION = 'Securities & Exchange Commission'
SOUTHERN_TRUST_COMPANY = 'Southern Trust Company'
TRUMP_ORG = 'Trump Organization'
UBS = 'UBS'
USANYS = 'USANYS'
US_ORG = 'US Office of Government Information Services'
VALAR_GLOBAL_FUND = 'Valar Global Fund'
VALAR_VENTURES = 'Valar Ventures'

# Locations
LITTLE_SAINT_JAMES = 'Little Saint James'

# Publications
BBC = 'BBC'
BLOOMBERG = 'Bloomberg'
CHINA_DAILY = "China Daily"
CNN = 'CNN'
DAILY_MAIL = 'Daily Mail'
DAILY_TELEGRAPH = "Daily Telegraph"
LA_TIMES = 'LA Times'
LEXIS_NEXIS = 'Lexis Nexis'
MIAMI_HERALD = 'Miami Herald'
NYT = "New York Times"
PALM_BEACH_DAILY_NEWS = f'{PALM_BEACH} Daily News'
PALM_BEACH_POST = f'{PALM_BEACH} Post'
SHIMON_POST = 'The Shimon Post'
THE_REAL_DEAL = 'The Real Deal'
VI_DAILY_NEWS = f'{VIRGIN_ISLANDS} Daily News'
WAPO = 'WaPo'


# Names to color white in the word counts
OTHER_NAMES = """
    adam al alain alan alison alfredo allen alex alexander amanda andres andrew angela ann anthony ari asia audrey
    back bard barrett barry ben bennet bernard bill black bo bob boris boyle brad brenner bruce bryan
    cameron capital caroline carolyn chris christian christina christopher cohen collins
    dan daniel danny darren dave david debbie department doj donald douglas dylan
    ed edward edwards elisa enforcement enterprise enterprises entourage epstein eric erik erika etienne
    faith fisher fitzgerald forget frances frank francesca fred friendly frost fuller fund
    gates gateway general gerald george gilbert girl glass gloria gold gordon greg
    haddad hall hanson harry hassan hay heather henry hill ho hoffman howard
    ian inc inc. ivan
    jack james jay jean jeanne jeff jeffrey jennifer jeremy jessica jim joel john jon jonathan joseph jr julia justin
    kafka kahn karl kate katherine kathryn kelly ken kevin krassner
    lang larry larsen laurie lawrence len leon les lesley linda link lisa london love
    management manhattan mann marc maria marie mark martin matthew melanie michael mike miller mitchell miles morris moskowitz
    nancy nathan neal new nicholas nicole nick norman ny
    oleg owen
    paul paula pen perry peter philip pierce plus police prince
    randall rangel reid richard robert roberts rodriguez roger rosenberg ross roth roy rubenstein rubin
    santos scott sean sergey service services simpson skip smith son stacey stanley stein stern stephen steve steven stone story susan
    team terry the thomas tim tom tony trust tyler
    unknown
    ventures victor viktor virginia
    wade waters
    aaron albert alberto alec alexandra alice anderson andre ann anna anne ariana arthur
    baldwin barack barrett ben benjamin berger bert binant bob bonner boyden bradley brady branson bright bruno bryant burton
    chapman charles charlie christopher clint cohen colin collins conway
    davis dean debbie debra deborah dennis diana diane diaz dickinson dixon dominique don dylan
    edmond elizabeth emily entwistle erik evelyn
    ferguson flachsbart francis franco frank
    gardner gary geoff geoffrey gilbert gloria goldberg gonzalez gould graham greene guarino gwyneth
    hancock harold harrison helen hirsch hofstadter horowitz hussein
    isaac isaacson
    jamie jane janet jason jeffrey jen jim joe johnson jones josh julie justin
    kathy kim kruger kyle
    lawrence leo leonard lenny leslie lieberman louis lynch lynn
    marcus marianne matt matthew melissa michele michelle moore moscowitz
    nancy nussbaum
    paulson philippe
    rafael ray richard richardson rob robert robin ron ross rubin rudolph ryan
    sara sarah seligman serge sergey silverman sloman smith snowden sorkin stanley steele stevie stewart
    ted theresa thompson tiffany timothy
    valeria
    walter warren weinstein weiss william
    zach zack
""".strip().split()


def constantize_name(name: str) -> str:
    if name == ANDRZEJ_DUDA:
        return 'ANDRZEJ_DUDA'
    elif name == MIROSLAV_LAJCAK:
        return 'MIROSLAV_LAJCAK'
    elif name == 'Paula Heil Fisher (???)':
        return 'PAULA'

    variable_name = remove_question_marks(name)
    variable_name = variable_name.removesuffix('.').removesuffix('Jr').replace('ź', 'z').replace('ø', 'o').strip()
    variable_name = variable_name.upper().replace('-', '_').replace(' ', '_').replace('.', '')

    if variable_name not in globals():
        return f"'{name}'"
    else:
        return variable_name


def extract_first_name(name: str) -> str:
    if ' ' not in name:
        return name

    return name.removesuffix(f" {extract_last_name(name)}")


def extract_last_name(name: str) -> str:
    if ' ' not in name:
        return name

    first_last_names = remove_question_marks(name).strip().split()

    if first_last_names[-1].startswith('Jr') and len(first_last_names[-1]) <= 3:
        return ' '.join(first_last_names[-2:])
    else:
        return first_last_names[-1]


def reverse_first_and_last_names(name: str) -> str:
    """If there's a comma in the name in the style 'Lastname, Firstname', reverse it and remove comma."""
    if '@' in name:
        return name.lower()

    if ', ' in name:
        names = name.split(', ')
        return f"{names[1]} {names[0]}"
    else:
        return name


def sort_names(names: Sequence[Name]) -> list[Name]:
    return sorted(names, key=lambda name: name or UNKNOWN)



"""
Methods to help with building various kinds of `DocCfg` or `EmailCfg` objects.
"""
from dateutil.parser import parse

from epstein_files.documents.documents.categories import Category, Interesting, Neutral, Uninteresting
from epstein_files.documents.config.doc_cfg import CommunicationCfg, DocCfg, DuplicateType, EmailCfg
from epstein_files.people.names import *
from epstein_files.util.helpers.string_helper import has_line_starting_with, join_truthy
from epstein_files.util.logging import logger

# Inferred category config regexes
DEUTSCHE_SOUTHERN_REGEX = re.compile(r"^.{,50}Deutsche Bank.{,1000}(SOUTHERN TRUST|RED HOOK QUARTER)", re.DOTALL)
EVIDENCE_REGEX = re.compile(r".{,1000}ITEM\s+WAS\s+NOT\s+SCANNED")
FBI_FILE_REGEX = re.compile(r"^(UNCLASSIFIED\s+)?FEDERAL BUREAU OF INVESTIGATION")
GRAND_JURY_REGEX = re.compile(r"Grand Jury", re.IGNORECASE)
HARD_DRIVE_REGEX = re.compile(r"westerndigital|Gold SATA")
LEGAL_FILING_REGEX = re.compile(r"^Case (\d+:\d+-.*?) Doc")
LSJE_FORM_REGEX = re.compile(r".{,5}LSJE,\s+LLC.*Emergency\s+Contact\s+Form", re.DOTALL)
SOUTHERN_FINANCIAL_REGEX = re.compile(r"^Southern Financial LLC salesforce.com")
SUBPOENA_REGEX = re.compile(r"GRAND JURY SUBPOENA")
VALAR_CAPITAL_CALL_REGEX = re.compile(r"^Val[ao]r.{,190} Capital Call", re.MULTILINE | re.IGNORECASE | re.DOTALL)
VI_DAILY_NEWS_REGEX = re.compile(r'virgin\s*is[kl][ai]nds\s*daily\s*news', re.IGNORECASE)

FBI_REPORT = f"report on Epstein investigation"  # (redacted)
JANE_DOE_V_USA = 'Jane Doe #1 and Jane Doe #2 v. United States'
LEDGERX_MSG = 'LedgerX was later acquired by FTX for $298 million'
WOLFF_EPSTEIN_ARTICLE_DRAFT = f"draft of an unpublished article ca. 2014"

DUPLICATES = {
    'EFTA00811539': ['EFTA00599617'],
    'EFTA00591792': ['EFTA00810358'],
}

CASE_IDS = {
    '1:20-cr-00330-AJN': f"US v. {GHISLAINE_MAXWELL}",
    '1:20-cv-00833-PAE': f"New York Times v. {BUREAU_OF_PRISONS}",
    '1:19-cr-00490-RMB': f"US v. {JEFFREY_EPSTEIN}",
    '1:22-cv-10904-JSR': f"US Virgin Islands v. JPMorgan Chase",
    '9:08-cv-80736-KAM': JANE_DOE_V_USA,
}

EMERGENCY_CONTACT_DATES = {
    'EFTA00003042': '2019-02-06'
}

DESCRIPTIONS = {
    'EFTA00015532': "alleging Epstein tried to buy victim's silence",
}


def build_cfg_from_text(doc: 'Document') -> DocCfg | None:
    """Scan the text to see if author, description, category, etc. can be derived from the contents."""
    text = doc.text
    lines = text.split('\n')
    cfg = None

    def _cfg(**kwargs) -> DocCfg:
        return DocCfg(id=doc.file_id, **kwargs)  # TODO: setting id to nothing sucks

    if FBI_FILE_REGEX.search(text):
        cfg = fbi_report(doc.file_id, 'memorandum or report')
    elif EVIDENCE_REGEX.search(text):
        cfg = _cfg(category=Neutral.LEGAL, description='photos of collected evidence')
    elif GRAND_JURY_REGEX.search(text[0:100]):
        cfg = _cfg(category=Neutral.LEGAL, description='grand jury proceedings')
    elif 'LedgerX' in text[0:500]:
        cfg = _cfg(category=Interesting.CRYPTO, description=LEDGERX_MSG)
    elif LSJE_FORM_REGEX.search(text):
        cfg = _cfg(
            category=Neutral.BUSINESS,
            date=EMERGENCY_CONTACT_DATES.get(doc.file_id, ''),
            description="emergency contact form for employee of Epstein's LSJE",
        )
    elif SUBPOENA_REGEX.search(text):
        cfg = _cfg(category=Neutral.LEGAL, description='grand jury subpoena or response')
    elif VI_DAILY_NEWS_REGEX.search(text):
        cfg = _cfg(category=Uninteresting.ARTICLE, author=VI_DAILY_NEWS)
    elif has_line_starting_with(text, [VALAR_GLOBAL_FUND, VALAR_VENTURES.upper()], 2):
        cfg = valar_cfg(doc.file_id, text=text)
    elif SOUTHERN_FINANCIAL_REGEX.match(text):
        cfg = _cfg(category=Interesting.MONEY, description="transactions by Epstein's Southern Financial LLC")
    elif DEUTSCHE_SOUTHERN_REGEX.match(text):
        cfg = _cfg(category=Interesting.MONEY, description=f"{DEUTSCHE_BANK} statement for Epstein's Southern Trust Company")
    elif VALAR_CAPITAL_CALL_REGEX.search(text):
        cfg = valar_cfg(doc.file_id, 'requesting money previously promised by Epstein to invest in a new opportunity')
    elif (case_match := LEGAL_FILING_REGEX.search(text)):
        case_name = CASE_IDS.get(case_match.group(1), f"case {case_match.group(1)}")
        description = join_truthy(f"legal filing in {case_name}", DESCRIPTIONS.get(doc.file_id, ''))
        cfg = _cfg(category=Neutral.LEGAL, description=description)
    elif len(text) < 2600 and HARD_DRIVE_REGEX.search(text):
        cfg = _cfg(category=Neutral.MISC, description='photo of a hard drive')
    elif lines[0].lower().strip() == 'valuation report':
        cfg = _cfg(category=Interesting.MONEY, description="valuations of Epstein's investments", is_interesting=True)

        try:
            cfg.date = str(parse(lines[1]))
        except Exception as e:
            logger.warning(f"Failed to parse valuation report date for derived DocCfg from {lines[0:2]}")

    if cfg:
        cfg.duplicate_ids = DUPLICATES.get(cfg.id, [])

    return cfg


def binant_redacted(id: str, truncate_to: int = 700) -> EmailCfg:
    return EmailCfg(id=id, truncate_to=truncate_to, description=f"redacted discussion of art advisor {ETIENNE_BINANT}")


def blaine_letter(id: str, date: str, suffix: str = '', **kwargs) -> CommunicationCfg:
    return letter(
        id,
        DAVID_BLAINE,
        date=date,
        description=join_truthy(f"recommending genius visa for a Epstein's assistant {SVETLANA_POZHIDAEVA}", suffix),
        is_interesting=True,
        recipients=['INS'],
        show_with_name=SVETLANA_POZHIDAEVA,
        **kwargs
    )


def fbi_defense_witness(id: str, witness: str, date: str = '') -> DocCfg:
    return DocCfg(
        id=id,
        author=FBI,
        date=date,
        description=f'Research and Key Findings for {witness}, defense witness for {GHISLAINE_MAXWELL}',
    )


def fedex_invoice(id: str, date: str) -> DocCfg:
    return DocCfg(id=id, author='FedEx', date=date, replace_text_with='FedEx invoice')


def fbi_report(id: str, description: str = FBI_REPORT, **kwargs) -> DocCfg:
    return DocCfg(id=id, author=FBI, category=Neutral.GOVERNMENT, description=description, **kwargs)


def important_messages_pad(id: str, date: str = '') -> DocCfg:
    return DocCfg(
        id=id,
        date=date,
        replace_text_with='"Important Message" formatted notepad with notes about missed phone calls etc.'
    )


def letter(id: str, author: str, recipients: list[Name], description: str, date: str = '', **kwargs) -> CommunicationCfg:
    return CommunicationCfg(
        id=id,
        author=author,
        category=Category.LETTER,
        date=date,
        description=description,
        recipients=recipients,
        **kwargs
    )


def phone_bill_cfg(id: str, author: str, dates: str = '', **kwargs) -> DocCfg:
    return DocCfg(
        id=id,
        author=author,
        category=Uninteresting.PHONE_BILL,
        replace_text_with=f"phone bill" + (f" covering {dates}" if dates else ''),
        **kwargs
    )


def shaher_murder_email(id: str, description: str = '', **kwargs) -> EmailCfg:
    return EmailCfg(
        id=id,
        description=join_truthy(description, f"discussion of the murder of Martine Vik Magnussen by {SHAHER_ABDULHAK_BESHER}'s son Farouk"),
        is_interesting=True,
        **kwargs
    )


def starr_letter(id: str, date: str, duplicate_ids: list[str], dupe_type: DuplicateType = 'same', **kwargs) -> CommunicationCfg:
    return letter(
        id=id,
        author=KEN_STARR,
        date=date,
        description="requesting lenient treatment for Epstein",
        duplicate_ids=duplicate_ids,
        dupe_type=dupe_type,
        recipients=['Judge Mark Filip'],
        **kwargs
    )


def valar_cfg(id: str, description: str = '', text: str = '') -> DocCfg:
    return DocCfg(
        id=id,
        category=Interesting.CRYPTO,  # TODO: not really crypto?
        author=VALAR_VENTURES,
        date='2015-06-30' if '6/30/2015' in text else '',
        description=description or f"is a fintech focused {PETER_THIEL} fund Epstein was invested in",
    )


def victim_diary(id: str, description: str) -> DocCfg:
    return DocCfg(id=id, category=Interesting.DIARY, description=description)


def whistleblower_cfg(id, description: str = '') -> EmailCfg:
    return EmailCfg(
        id=id,
        description=join_truthy('SEC whistleblower email', description, ', '),
        is_interesting=True,
    )


def wolff_draft_cfg(id: str, suffix: str = '', **kwargs) -> DocCfg:
    return DocCfg(
        id=id,
        author=MICHAEL_WOLFF,
        date='2014-08-31' if 'show_with_name' in kwargs else '2014-09-01',  # Very approximate, make excerpt show first
        description=join_truthy(WOLFF_EPSTEIN_ARTICLE_DRAFT, suffix),
        **kwargs
    )



import logging
from dataclasses import asdict, dataclass, field
from hashlib import md5
from pathlib import Path
from typing import Mapping

from rich.text import Text

from epstein_files.output.rich import join_texts, prefix_with, styled_dict
from epstein_files.util.constant.strings import ALT_LINK_STYLE, ARCHIVE_LINK_COLOR, DOJ_DATASET_ID_REGEX, LEON_BLACK_EMAIL_ID
from epstein_files.util.constant.urls import *
from epstein_files.util.env import DOJ_PDFS_20260130_DIR, site_config
from epstein_files.util.helpers.file_helper import (coerce_file_stem, coerce_url_slug, extract_file_id,
     extract_efta_id, file_size, file_size_to_str, is_doj_file, is_house_oversight_file, is_local_extract_file,
     open_file_or_url)
from epstein_files.util.helpers.link_helper import link_text_obj, parenthesize
from epstein_files.util.helpers.rich_helpers import no_bold
from epstein_files.util.logging import logger

FILE_PROPS = [
    'file_id',
    'file_size',
    'filename',
    'is_doj_file',
    'is_local_extract_file',
    'url_slug',
]


@dataclass
class FileInfo:
    """
    Attributes:
        local_path (Path): local path of the document's underlying .txt file
        doj_2026_dataset_id (int, optional): DataSet number for DOJ files
        file_id (str): file_id (str): ID string - 6 numbers with zero padding for HOUSE_OVERSIGHT, full EFTAXXXXX for DOJ files
    """
    local_path: Path
    doj_2026_dataset_id: int | None = None
    file_id: str = field(init=False)

    def __post_init__(self):
        self.file_id = extract_file_id(self.local_path)

        # Extract the DOJ dataset ID from the path
        if self.is_doj_file:
            if (data_set_match := DOJ_DATASET_ID_REGEX.search(str(self.local_path))):
                self.doj_2026_dataset_id = int(data_set_match.group(1))
                logger.debug(f"Extracted data set ID {self.doj_2026_dataset_id} for {self.url_slug}")
            else:
                self.warn(f"Couldn't find a data set ID in path '{self.local_path}'! Cannot create valid links.")

    @property
    def as_dict(self) -> Mapping[str, str | Path]:
        props = {k: v for k, v in asdict(self).items() if v}
        props.update({k: getattr(self, k) for k in FILE_PROPS if getattr(self, k)})
        props.update(self.paths)
        props.update(self.urls)

        if self.url_slug != self.file_id:
            props['url_slug'] = self.url_slug

        return props

    @property
    def efta_id(self) -> int:
        if self.is_doj_file:
            return extract_efta_id(self.file_id)
        else:
            raise ValueError(f"{self.file_id} is not an EFTA prefix file!")

    @property
    def external_url(self) -> str:
        """The primary external URL to use when linking to this document's source."""
        if self.is_doj_file and self.doj_2026_dataset_id:
            return doj_2026_file_url(self.doj_2026_dataset_id, self.url_slug)
        elif self.is_eml_file:
            return JMAIL_URL
        else:
            return self.url_for_id(self.url_slug)

    @property
    def file_size(self) -> int:
        try:
            return file_size(self.local_path)
        except FileNotFoundError as e:
            if self.has_file:
                self.warn(str(e))
            else:
                self.log(f"no underlying file (but expected): {e}")

            return -1

    @property
    def file_size_str(self) -> str:
        return file_size_to_str(self.file_size)

    @property
    def file_stem(self) -> str:
        if self.is_eml_file:
            return self.local_path.stem
        else:
            return coerce_file_stem(self.file_id)

    @property
    def filename(self) -> str:
        return self.local_path.name

    @property
    def has_file(self) -> bool:
        """Returns false for derivations of massive Dilorio emails."""
        return not ((LEON_BLACK_EMAIL_ID in self.file_id or self.is_doj_file) and self.is_local_extract_file)

    @property
    def is_eml_file(self) -> bool:
        return str(self.local_path).endswith('.eml')

    @property
    def is_doj_file(self) -> bool:
        return is_doj_file(self.file_stem)

    @property
    def is_house_oversight_file(self) -> bool:
        return is_house_oversight_file(self.file_stem)

    @property
    def is_local_extract_file(self) -> bool:
        """True if extracted from other file (identifiable from filename e.g. HOUSE_OVERSIGHT_012345_1.txt)."""
        return is_local_extract_file(self.filename)

    @property
    def local_pdf_path(self) -> Path | None:
        """Path to the source PDF (only applies to DOJ files that were manually extracted)."""
        if self.is_doj_file and DOJ_PDFS_20260130_DIR:
            return next((p for p in DOJ_PDFS_20260130_DIR.glob('**/*.pdf') if p.stem == self.file_stem), None)

    @property
    def paths(self) -> Mapping[str, Path]:
        return {k: Path(v) for k, v in self._props_with_suffix('path').items() if v}

    @property
    def url_slug(self) -> str:
        if self.is_eml_file:
            hash = md5(self.local_path.read_bytes()).hexdigest()
            logger.warning(f"{self.file_stem} hash: {hash}")
            return hash

        return coerce_url_slug(self.file_id)

    @property
    def urls(self) -> Mapping[str, str]:
        urls = {k: str(v) for k, v in self._props_with_suffix('url').items() if v}
        urls = {k: (v if v.startswith('http') else f"https://{v}") for k, v in urls.items()}
        return urls

    def epsteinify_link(self, style: str = ARCHIVE_LINK_COLOR, link_txt: str = '') -> Text:
        return self._build_link(epsteinify_doc_url, style, link_txt)

    def epstein_media_link(self, style: str = ARCHIVE_LINK_COLOR, link_txt: str = '') -> Text:
        return self._build_link(epstein_media_doc_url, style, link_txt)

    def epstein_web_link(self, style: str = ARCHIVE_LINK_COLOR, link_txt: str = '') -> Text:
        return self._build_link(epstein_web_doc_url, style, link_txt)

    def rollcall_link(self, style: str = ARCHIVE_LINK_COLOR, link_txt: str = '') -> Text:
        return self._build_link(rollcall_doc_url, style, link_txt)

    def external_link_markup(self, style: str = '', id_only: bool = False) -> str:
        link_txt = self.file_id if id_only else self.file_stem
        return link_markup(self.external_url, link_txt, style=no_bold(style))

    def external_link_txt(self, style: str = '', id_only: bool = False) -> Text:
        id_only = id_only or self.is_eml_file
        return Text.from_markup(self.external_link_markup(style, id_only))

    def build_external_links(self, style: str = '', with_alt_links: bool = False, id_only: bool = False) -> Text:
        """Returns colored links to epstein.media and alternates in a Text object."""
        links = [self.external_link_txt(style, id_only)]

        if with_alt_links and not self.is_eml_file:
            if self.doj_2026_dataset_id:
                jmail_url = jmail_doj_2026_file_url(self.doj_2026_dataset_id, self.file_id)
                jmail_link = link_text_obj(jmail_url, JMAIL, style=f"{style} dim" if style else ARCHIVE_LINK_COLOR)
                links.append(jmail_link)
            else:
                links.append(self.epsteinify_link(style=ALT_LINK_STYLE, link_txt=EPSTEINIFY))
                links.append(self.epstein_web_link(style=ALT_LINK_STYLE, link_txt=EPSTEIN_WEB))

                # TODO: pass the Document so we can reinstate this?
                # if self._class_name == 'Email':
                #     links.append(self.rollcall_link(style=ALT_LINK_STYLE, link_txt=ROLLCALL))

        links = links if site_config.max_alt_links is None else links[0:site_config.max_alt_links + 1]
        links = [links[0]] + [parenthesize(link) for link in links[1:]]
        base_txt = Text('', style='white' if with_alt_links else ARCHIVE_LINK_COLOR)
        return base_txt.append(join_texts(links))

    def log(self, msg: str, level: int = logging.INFO):
        """Log a message with with this document's filename as a prefix."""
        logger.log(level, f"{self.file_stem} {msg}")

    def open(self) -> None:
        open_file_or_url(self.local_path)

    def open_pdf(self) -> None:
        if self.local_pdf_path:
            open_file_or_url(self.local_pdf_path)
        else:
            raise RuntimeError(f"No PDF for {self.file_stem}!")

    def warn(self, msg: str) -> None:
        """Print a warning message prefixed by info about this `file_id`."""
        self.log(msg, level=logging.WARNING)

    def _build_link(self, fxn: Callable[[str], str], style: str = ARCHIVE_LINK_COLOR, link_txt: str = '') -> Text:
        return link_text_obj(fxn(self.url_slug), link_txt or self.file_stem, style)

    def _props_with_suffix(self, suffix: str) -> Mapping[str, str | Path]:
        return {k: getattr(self, k) for k in dir(self) if k.endswith(suffix)}

    def __rich__(self) -> Text:
        """Text obj with local paths and URLs."""
        txt_lines = styled_dict(self.as_dict, sep=': ')
        return prefix_with(txt_lines, ' ', pfx_style='grey', indent=2)

    def __str__(self) -> str:
        lines = [f"{k:>40}: {v}" for k, v in asdict(self).items() if k != 'file_id']
        return '\n'.join(lines)

    @classmethod
    def url_for_id(cls, url_slug: str) -> str:
        """Exists to codify the choice of which site to prefer. TODO: this doesn't work for DOJ files..."""
        return epstein_media_doc_url(url_slug)



from epstein_files.documents.config.doc_cfg import DocCfg, EmailCfg
from epstein_files.people.names import *
from epstein_files.util.constant.strings import *

FIRE_AND_FURY = f"Fire And Fury"
PATTERSON_BOOK_SCANS = f'pages of "Filthy Rich: The Shocking True Story of {JEFFREY_EPSTEIN}"'


BOOK_CFGS = [
    DocCfg(id='017088', author=ALAN_DERSHOWITZ, description=f'"Taking the Stand: My Life in the Law" (draft)', date='2012-04-02'),  # Slow file
    DocCfg(id='013501', author='Arnold J. Mandell', description=f'The Nearness Of Grace: A Personal Science Of Spiritual Transformation', date='2005-01-01'),
    DocCfg(id='012899', author='Ben Goertzel', description=f'Engineering General Intelligence: A Path to Advanced AGI Via Embodied Learning and Cognitive Synergy', date='2013-09-19'),
    DocCfg(id='018438', author='Clarisse Thorn', description=f'The S&M Feminist', date='2012-05-14'),
    DocCfg(id='019477', author=EDWARD_JAY_EPSTEIN, description=f'How America Lost Its Secrets: Edward Snowden, the Man, and the Theft'),
    DocCfg(id='020153', author=EDWARD_JAY_EPSTEIN, description=f'The Snowden Affair: A Spy Story In Six Parts'),
    DocCfg(id='011472', author=EHUD_BARAK, description=f'"Night Flight" (draft)', date='2006-07-12', duplicate_ids=['027849'], is_interesting=False),  # date from _extract_timestamp()
    DocCfg(id='010912', author=GORDON_GETTY, description=f'"Free Growth and Other Surprises" (draft)', date='2018-10-18'),
    DocCfg(id='010477', author=JAMES_PATTERSON, description=PATTERSON_BOOK_SCANS, date='2016-10-10'),
    DocCfg(id='010486', author=JAMES_PATTERSON, description=PATTERSON_BOOK_SCANS, date='2016-10-10'),
    DocCfg(id='021958', author=JAMES_PATTERSON, description=PATTERSON_BOOK_SCANS, date='2016-10-10'),
    DocCfg(id='022058', author=JAMES_PATTERSON, description=PATTERSON_BOOK_SCANS, date='2016-10-10'),
    DocCfg(id='022118', author=JAMES_PATTERSON, description=PATTERSON_BOOK_SCANS, date='2016-10-10'),
    DocCfg(id='019111', author=JAMES_PATTERSON, description=PATTERSON_BOOK_SCANS, date='2016-10-10'),
    DocCfg(id='015675', author='James Tagg', description=f'Are the Androids Dreaming Yet? Amazing Brain Human Communication, Creativity & Free Will', date='2015-06-01'),  # Slow file
    DocCfg(id='016804', author=JOHN_BROCKMAN, description='Deep Thinking: Twenty-Five Ways of Looking at AI', date='2019-02-19', duplicate_ids=['016221']),
    DocCfg(id='018232', author='Joshua Cooper Ramo', description=f'The Seventh Sense: Power, Fortune & Survival in the Age of Networks', date='2015-03-11'),
    DocCfg(id='012747', author='Marc D. Hauser', description=f'Evilicious: Explaining Our Taste For Excessive Harm'),
    DocCfg(id='032724', author=MICHAEL_WOLFF, description=f'cover of "{FIRE_AND_FURY}"', date='2018-01-05', is_interesting=False),
    DocCfg(id='021120', author=MICHAEL_WOLFF, description=f'chapter of "Siege: Trump Under Fire"'),
    DocCfg(id='019874', author=MICHAEL_WOLFF, description=FIRE_AND_FURY, date='2018-01-05', is_interesting=False),
    DocCfg(id='015032', author=PAUL_KRASSNER, description=f"60 Years of Investigative Satire: The Best of {PAUL_KRASSNER}"),
    DocCfg(id='023731', author=ROGER_SCHANK, description=f'Teaching Minds How Cognitive Science Can Save Our Schools'),
    DocCfg(id='021247', author='The Chicago Social Brain Network', description=f'Invisible Forces And Powerful Beliefs: Gravity, Gods, And Minds', date='2010-10-04'),
    DocCfg(id='013796', author='Tim Ferriss', description=f'The 4-Hour Workweek'),
    DocCfg(id='021145', author=VIRGINIA_GIUFFRE, description=f'"The Billionaire\'s Playboy Club" (draft?)'),
    DocCfg(id='031533', description=f'pages from a book about the Baylor University sexual assault scandal and Sam Ukwuachu'),
    # DOJ files
    DocCfg(id='EFTA00008020', replace_text_with='"Massage for Dummies"'),
    DocCfg(id='EFTA00008320', replace_text_with='"Massage for Dummies (???)"'),
    DocCfg(id='EFTA00008220', replace_text_with='"Massage book: Chapter 11: Putting the Moves Together"'),
    DocCfg(id='EFTA00008120', replace_text_with='"Part II: The Art of Receiving a Massage"'),
]



"""
Skype conversation logs.
"""
from epstein_files.documents.config.doc_cfg import CommunicationCfg
from epstein_files.people.names import *
from epstein_files.util.constant.strings import *


SKYPE_LOG_CFGS = [
    CommunicationCfg(id='032206', recipients=[LAWRENCE_KRAUSS], is_interesting=False),
    CommunicationCfg(id='032208', recipients=[LAWRENCE_KRAUSS], is_interesting=False),
    CommunicationCfg(id='032209', recipients=[LAWRENCE_KRAUSS], is_interesting=False),
    CommunicationCfg(id='032210', recipients=['linkspirit'], is_interesting=True),
    CommunicationCfg(id='018224', recipients=['linkspirit', LAWRENCE_KRAUSS], is_interesting=True),  # we don't know who linkspirit is yet
    CommunicationCfg(id='EFTA01617727'),
    CommunicationCfg(
        id='EFTA00507334',
        recipients=[
            'Aleksandra Eriksson',
            ANASTASIYA_SIROOCHENKO,
            'Catherine',
            DANIEL_SIAD,
            DAVID_STERN,
            EMAD_HANNA,
            'Jade Huang',
            NADIA_MARCINKO,
            'sexysearch2010',
            'sophiembh'
        ],
    ),
    CommunicationCfg(id='EFTA01613143', author=MELANIE_WALKER, date='2017-06-24'),
    CommunicationCfg(id='EFTA01217787', recipients=[TYLER_SHEARS, 'Hanna Traff at Spotify'], is_interesting=True),
    CommunicationCfg(id='EFTA01217703', recipients=['actress Athena Zelcovich', JOSCHA_BACH, LAWRENCE_KRAUSS]),
    CommunicationCfg(id='EFTA01217736', recipients=['actress Athena Zelcovich', TYLER_SHEARS]),
    CommunicationCfg(
        id='EFTA01623342',
        author=ANNA_KASATKINA,
        author_uncertain='https://www.reddit.com/r/Epstein/comments/1qwbn5i/trafficker_julia_santos/',
        description='recruiting russian girls',
        is_interesting=True,
    ),
]



from epstein_files.documents.config.config_builder import blaine_letter
from epstein_files.documents.config.doc_cfg import CommunicationCfg
from epstein_files.people.names import LEON_BLACK


# This category makes is_interesting default to True
LETTER_CFGS = [
    CommunicationCfg(
        id='026011',
        author='Gennady Mashtalyar',
        date='2016-06-24',  # date is based on Brexit reference but he could be backtesting,
        description=f"about algorithmic trading",
    ),
    CommunicationCfg(id='026134', recipients=['George'], description=f'about opportunities to buy banks in Ukraine'),
    blaine_letter(id='019086', date='2015-05-27', suffix='naming various Putin puppet regimes', show_full_panel=True),
    blaine_letter(id='019474', date='2015-05-29'),
    blaine_letter(id='019476', date='2015-06-01'),

    # DOJ files
    CommunicationCfg(id='EFTA00007609', recipients=['Alberto'], duplicate_ids=['EFTA00007582']),
    CommunicationCfg(id='EFTA02731023', author='Senator Ron Wyden', recipients=[LEON_BLACK], is_interesting=False),
    CommunicationCfg(id='EFTA02731018', author='Senator Ron Wyden', recipients=['Marc Rowan'], is_interesting=False),
]



import re
from collections import defaultdict
from copy import deepcopy

from rich.console import Console
from rich.highlighter import RegexHighlighter
from rich.text import Text

from epstein_files.output.highlighted_names import HighlightedNames
from epstein_files.output.highlight_config import HIGHLIGHT_GROUPS, HIGHLIGHTED_NAMES
from epstein_files.util.constant.strings import HIGHLIGHTED_QUOTE, JEE, REGEX_STYLE_PREFIX
from epstein_files.util.env import args
from epstein_files.util.helpers.data_helpers import sort_dict
from epstein_files.util.logging import logger

WEAK_DATE_REGEX = re.compile(r"^(\d\d?/|20|http|On ).*")


class EpsteinHighlighter(RegexHighlighter):
    """Finds and colors interesting keywords based on the above config."""
    base_style = f"{REGEX_STYLE_PREFIX}."
    highlights: list[re.Pattern] = [highlight_group.regex for highlight_group in HIGHLIGHT_GROUPS]
    highlight_counts = defaultdict(int)

    def highlight(self, text: Text) -> None:
        """overrides https://rich.readthedocs.io/en/latest/_modules/rich/highlighter.html#RegexHighlighter"""
        highlight_regex = text.highlight_regex

        for re_highlight in self.highlights:
            highlight_regex(re_highlight, style_prefix=self.base_style)

            if args.stats and isinstance(re_highlight, re.Pattern):
                # if re_highlight.search(text.plain):
                #     logger.debug(f"Matched pattern {re_highlight.pattern}")

                for match in re_highlight.finditer(text.plain):
                    matched_str = (match.group(1) or 'None').replace('\n', ' ').strip().lower()
                    # logger.debug(f"Matched string {match.group(1)}")
                    type(self).highlight_counts[matched_str] += 1

    def print_highlight_counts(self, console: Console) -> None:
        """Print counts of how many times strings were highlighted."""
        highlight_counts = deepcopy(self.highlight_counts)

        for highlighted, count in sort_dict(highlight_counts):
            if highlighted is None or WEAK_DATE_REGEX.match(highlighted):
                continue

            try:
                console.print(f"{highlighted:25s} highlighted {count} times")
            except Exception as e:
                logger.error(f"Failed to print highlight count {count} for {highlighted}")


class NonEpsteinHighlighter(EpsteinHighlighter):
    """Doesn't highlight Epstein's name, highlights only `HighlightedNames` patterns (no `HighlightPatterns`)."""
    highlights: list[re.Pattern] = [hn.regex for hn in HIGHLIGHTED_NAMES if hn.label not in [JEE]]


def temp_highlighter(pattern: str, theme_style_name: str = HIGHLIGHTED_QUOTE) -> EpsteinHighlighter:
    """Temporary highlighter that adds `pattern` to the usual highlight regexes."""
    class TempHighlighter(EpsteinHighlighter):
        highlights = [re.compile(fr"(?P<{theme_style_name}>{pattern})", re.IGNORECASE)] + EpsteinHighlighter.highlights

    return TempHighlighter()


highlighter = EpsteinHighlighter()
non_epstein_highlighter = NonEpsteinHighlighter()
